﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ParameterlessPopulationPyramid {
  [Item("Deceptive Trap Problem", "Genome encodes completely separable blocks, where each block is fully deceptive.")]
  [StorableClass]
  [Creatable("Parameterless Population Pyramid")]
  public class DeceptiveTrapProblem : BinaryVectorProblem {
    [StorableConstructor]
    protected DeceptiveTrapProblem(bool deserializing) : base(deserializing) { }
    protected DeceptiveTrapProblem(DeceptiveTrapProblem original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DeceptiveTrapProblem(this, cloner);
    }

    public override bool Maximization {
      get { return true; }
    }

    private const string TrapSizeParameterName = "Trap Size";

    public IFixedValueParameter<IntValue> TrapSizeParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters[TrapSizeParameterName]; }
    }

    public int TrapSize {
      get { return TrapSizeParameter.Value.Value; }
      set { TrapSizeParameter.Value.Value = value; }
    }
    public DeceptiveTrapProblem()
      : base() {
      Parameters.Add(new FixedValueParameter<IntValue>(TrapSizeParameterName, "", new IntValue(5)));
    }

    public override double Evaluate(bool[] individual) {
      if (individual.Length != Length) throw new ArgumentException("The individual has not the correct length.");
      int total = 0;
      for (int i = 0; i < individual.Length; i += TrapSize) {
        int partial = 0;
        for (int index = i; index < i + TrapSize; index++) {
          if (individual[index]) partial++;
        }

        // Make it deceptive
        if (partial < TrapSize) {
          partial = TrapSize - partial - 1;
        }
        total += partial;
      }
      return total;
    }
  }
}
